package com.mortals.xhx.modules.information.task;

import cn.hutool.core.date.DateTime;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.ITask;
import com.mortals.framework.service.ITaskExcuteService;
import com.mortals.xhx.common.code.NodeTypeEnum;
import com.mortals.xhx.common.utils.SnowFlakeUtil;
import com.mortals.xhx.modules.information.model.InformationEntity;
import com.mortals.xhx.modules.information.service.InformationService;
import com.mortals.xhx.modules.records.model.RecordEntity;
import com.mortals.xhx.modules.records.service.RecordService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Wz
 * @Date: 2021-08-2021/8/3-16:28
 * @Description:办件数据推送省一体化平台（审查）
 */
@Slf4j
@Component("reviewInformationTaskImpl")
public class ReviewInformationTaskImpl implements ITaskExcuteService {
    @Autowired
    private InformationService informationService;

    @Autowired
    private RecordService recordService;

    @Value("${egov.examineNode-url}")
    private String url;

    @Override
    public void excuteTask(ITask task) throws AppException {
        //组装申请并受理数据推送一体化平台
        this.reviewData();
    }

    @Override
    public void stopTask(ITask task) throws AppException {

    }

    private void reviewData() {
        InformationEntity informationEntity = new InformationEntity();
        informationEntity.setNodeType(NodeTypeEnum.审查.getCode());
        //查询转化表的数据
        List<InformationEntity> informationEntities = informationService.find(informationEntity, null);
        if (informationEntities.size() > 0) {
            informationEntities.forEach(e -> {
                HashMap<String, Object> dataMap = new HashMap<>(16);
                dataMap.put("operateType", "1");
                HashMap<String, Object> targetConditionDefMap = new HashMap<>(16);
                targetConditionDefMap.put("provinceAreaCode", e.getAreaCode());
                targetConditionDefMap.put("deptCode", e.getDeptCode());
                targetConditionDefMap.put("areaCode", e.getAreaCode());
                dataMap.put("targetConditionDef", targetConditionDefMap);

                HashMap<String, Object> baseInfoMap = new HashMap<>(16);
                baseInfoMap.put("reCode", e.getEventCode());
                baseInfoMap.put("code", e.getOfficeCode());
                baseInfoMap.put("applyCode", "");
                baseInfoMap.put("sysCode", e.getSystemCode());
                baseInfoMap.put("bizStatus", e.getBizStatus());
                dataMap.put("baseInfo", baseInfoMap);

                HashMap<String, Object> valueMap = new HashMap<>(16);
                HashMap<String, Object> rlogisticsinfoMap = new HashMap<>(16);
                rlogisticsinfoMap.put("nodeType", e.getNodeType());
                /*
                 * 测试数据--start
                 * */
                rlogisticsinfoMap.put("addUserId", "HCPSH_ZSJ");
                rlogisticsinfoMap.put("curHandlerName", "郑世杰");
                rlogisticsinfoMap.put("nodeIdCard", "51372319980502619x");
                rlogisticsinfoMap.put("nodePhone", "17158320000");
                rlogisticsinfoMap.put("nodeRegionId", e.getAreaCode());
                rlogisticsinfoMap.put("nodeOrganId", "513723199805026192");
                rlogisticsinfoMap.put("addOrganId", e.getDeptCode());
                rlogisticsinfoMap.put("addOrganName", "测试单位名称，正式上线后改动");
                /*
                 * 测试数据-finsh
                 * */
                rlogisticsinfoMap.put("auditAdvice", "1");
                rlogisticsinfoMap.put("auditRemark", "同意");
                rlogisticsinfoMap.put("genTime", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(e.getGenTime()));
                rlogisticsinfoMap.put("completeTime", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(e.getCompleteTime()));
                valueMap.put("rprojectruntask", rlogisticsinfoMap);
                dataMap.put("value", valueMap);
                String post = "";
                JSONObject ifResultInfo = null;
                try {
                    post = HttpUtil.post(url, JSONObject.toJSONString(dataMap));
                    ifResultInfo = JSONObject.parseObject(post).getJSONObject("ifResultInfo");
                    if ("200".equals(ifResultInfo.getString("code"))) {
                        e.setBizStatus(NodeTypeEnum.决定.getDesc());
                        e.setNodeType(NodeTypeEnum.决定.getCode());
                        e.setProcessStatusName("决定");
                        informationService.update(e, null);
                        log.info("推送一体化审查数据成功，");
                    } else {
                        e.setFailTimes(e.getFailTimes() + 1);
                        e.setFailReason(ifResultInfo.getString("message"));
                        informationService.update(e, null);
                        log.info("省一体化申请并受理数据推送失败：" + ifResultInfo.getString("message"));
                    }
                    //记录接口交换记录
                    recordInterfaceExchangeRecord(post, ifResultInfo, e, dataMap);
                } catch (Exception ex) {
                    log.error("省一体化审查数据推送失败：连接异常" + post);
                    //记录接口交换记录
                    recordInterfaceExchangeRecord(post, ifResultInfo, e, dataMap);

                }
            });
        } else {
            log.info("没有从办件表中获取到审查数据");
        }
    }

    private void recordInterfaceExchangeRecord(String post, JSONObject ifResultInfo, InformationEntity e, Map<String, Object> dataMap) {
        RecordEntity recordEntity = new RecordEntity();
        recordEntity.setId(SnowFlakeUtil.get().nextId());
        recordEntity.setPieceId(e.getId());
        recordEntity.setCurrentStepName("决定");
        recordEntity.setCurrentStepCode(NodeTypeEnum.决定.getCode());
        recordEntity.setIncomingMessage(JSONObject.toJSONString(dataMap));
        recordEntity.setReturnMessage(JSONObject.toJSONString(post));
        recordEntity.setInterfaceCallTime(DateTime.now());
        recordEntity.setInterfaceCallStatus(ifResultInfo.getString("code"));
        recordEntity.setSystemCode(e.getSystemCode());
        recordService.save(recordEntity, null);
    }
}
